@@ -8,7 +8,7 @@
 
 #define G_LOG_DOMAIN "phosh-wayland"
 
-#include "config.h"
+#include "phosh-config.h"
 #include "phosh-enums.h"
 #include "phosh-wayland.h"
 
@@ -51,6 +51,7 @@ struct _PhoshWayland {
   struct zwlr_output_power_manager_v1 *zwlr_output_power_manager_v1;
   struct zxdg_output_manager_v1 *zxdg_output_manager_v1;
   struct zwlr_screencopy_manager_v1 *zwlr_screencopy_manager_v1;
+  struct zphoc_layer_shell_effects_v1 *zphoc_layer_shell_effects_v1;
   struct wl_shm *wl_shm;
   GHashTable *wl_outputs;
   PhoshWaylandSeatCapabilities seat_capabilities;
@@ -76,12 +77,18 @@ registry_handle_global (void *data,
       &phosh_private_interface,
       MIN(6, version));
     self->phosh_private_version = version;
+  } else if (!strcmp (interface, zphoc_layer_shell_effects_v1_interface.name)) {
+    self->zphoc_layer_shell_effects_v1 = wl_registry_bind (
+      registry,
+      name,
+      &zphoc_layer_shell_effects_v1_interface,
+      1);
   } else if (!strcmp (interface, zwlr_layer_shell_v1_interface.name)) {
     self->layer_shell = wl_registry_bind (
       registry,
       name,
       &zwlr_layer_shell_v1_interface,
-      1);
+      2);
   } else if (!strcmp (interface, "wl_output")) {
     output = wl_registry_bind (
       registry,
@@ -265,18 +272,21 @@ phosh_wayland_constructed (GObject *object)
   num_outputs = g_hash_table_size(self->wl_outputs);
   if (!num_outputs || !self->layer_shell || !self->idle_manager ||
       !self->input_inhibit_manager || !self->xdg_wm_base ||
-      !self->zxdg_output_manager_v1) {
-    g_error ("Could not find needed globals\n"
+      !self->zxdg_output_manager_v1 ||
+      !self->zphoc_layer_shell_effects_v1) {
+    g_error ("Wayland compositor lacks needed globals\n"
              "outputs: %d, layer_shell: %p, idle_manager: %p, "
              "inhibit: %p, xdg_wm: %p, "
              "xdg_output: %p, wlr_output_manager: %p, "
-             "wlr_foreign_toplevel_manager: %p"
+             "wlr_foreign_toplevel_manager: %p, "
+             "zphoc_layer_shell_effects_v1: %p"
              "\n",
              num_outputs, self->layer_shell, self->idle_manager,
              self->input_inhibit_manager, self->xdg_wm_base,
              self->zxdg_output_manager_v1,
              self->zwlr_output_manager_v1,
-             self->zwlr_foreign_toplevel_manager_v1);
+             self->zwlr_foreign_toplevel_manager_v1,
+             self->zphoc_layer_shell_effects_v1);
   }
   if (!self->phosh_private) {
     g_info ("Could not find phosh private interface, disabling some features");
@@ -307,6 +317,7 @@ phosh_wayland_dispose (GObject *object)
   g_clear_pointer (&self->zwlr_screencopy_manager_v1, zwlr_screencopy_manager_v1_destroy);
   g_clear_pointer (&self->zwp_virtual_keyboard_manager_v1, zwp_virtual_keyboard_manager_v1_destroy);
   g_clear_pointer (&self->zxdg_output_manager_v1, zxdg_output_manager_v1_destroy);
+  g_clear_pointer (&self->zphoc_layer_shell_effects_v1, zphoc_layer_shell_effects_v1_destroy);
 
   g_clear_pointer (&self->wl_outputs, g_hash_table_destroy);
 
@@ -552,3 +563,12 @@ phosh_wayland_get_seat_capabilities (PhoshWayland *self)
 
   return self->seat_capabilities;
 }
+
+
+struct zphoc_layer_shell_effects_v1 *
+phosh_wayland_get_zphoc_layer_shell_effects_v1 (PhoshWayland *self)
+{
+  g_return_val_if_fail (PHOSH_IS_WAYLAND (self), NULL);
+
+  return self->zphoc_layer_shell_effects_v1;
+}
