@@ -8,7 +8,7 @@
 
 #define G_LOG_DOMAIN "phosh-quick-setting"
 
-#include "config.h"
+#include "phosh-config.h"
 
 #include "status-icon.h"
 #include "quick-setting.h"
