@@ -14,6 +14,7 @@
 #include "login1-manager-dbus.h"
 #include "login1-session-dbus.h"
 #include "lockscreen-manager.h"
+#include "session-presence.h"
 #include "util.h"
 
 /**
@@ -39,26 +40,28 @@ enum {
 };
 static GParamSpec *props[PROP_LAST_PROP];
 
-static void phosh_screen_saver_manager_screen_saver_iface_init (
-  PhoshScreenSaverDBusScreenSaverIface *iface);
+static void phosh_screen_saver_manager_screen_saver_iface_init (PhoshDBusScreenSaverIface *iface);
 
 typedef struct _PhoshScreenSaverManager
 {
-  PhoshScreenSaverDBusScreenSaverSkeleton parent;
+  PhoshDBusScreenSaverSkeleton parent;
 
   int idle_id;
   int dbus_name_id;
   PhoshLockscreenManager *lockscreen_manager;
+  PhoshSessionPresence *presence;  /* gnome-session's presence interface */
 
-  PhoshLogin1SessionDBusLoginSession *logind_session_proxy;
-  PhoshLogin1ManagerDBusLoginManager *logind_manager_proxy;
+  PhoshDBusLoginSession *logind_session_proxy;
+  PhoshDBusLoginManager *logind_manager_proxy;
+
+  GCancellable          *cancel;
 } PhoshScreenSaverManager;
 
 G_DEFINE_TYPE_WITH_CODE (PhoshScreenSaverManager,
                          phosh_screen_saver_manager,
-                         PHOSH_SCREEN_SAVER_DBUS_TYPE_SCREEN_SAVER_SKELETON,
+                         PHOSH_DBUS_TYPE_SCREEN_SAVER_SKELETON,
                          G_IMPLEMENT_INTERFACE (
-                           PHOSH_SCREEN_SAVER_DBUS_TYPE_SCREEN_SAVER,
+                           PHOSH_DBUS_TYPE_SCREEN_SAVER,
                            phosh_screen_saver_manager_screen_saver_iface_init));
 
 static void
@@ -81,9 +84,9 @@ phosh_screen_saver_manager_set_property (GObject *object,
 
 
 static void
-phosh_screen_saver_manager_get_property (GObject *object,
-                                         guint property_id,
-                                         GValue *value,
+phosh_screen_saver_manager_get_property (GObject    *object,
+                                         guint       property_id,
+                                         GValue     *value,
                                          GParamSpec *pspec)
 {
   PhoshScreenSaverManager *self = PHOSH_SCREEN_SAVER_MANAGER (object);
@@ -99,8 +102,8 @@ phosh_screen_saver_manager_get_property (GObject *object,
 }
 
 static gboolean
-handle_get_active (PhoshScreenSaverDBusScreenSaver *skeleton,
-                   GDBusMethodInvocation           *invocation)
+handle_get_active (PhoshDBusScreenSaver  *skeleton,
+                   GDBusMethodInvocation *invocation)
 {
   PhoshScreenSaverManager *self = PHOSH_SCREEN_SAVER_MANAGER (skeleton);
   gboolean locked;
@@ -111,16 +114,15 @@ handle_get_active (PhoshScreenSaverDBusScreenSaver *skeleton,
   locked = phosh_lockscreen_manager_get_locked (self->lockscreen_manager);
   g_debug ("DBus call GetActive: %d", locked);
 
-  phosh_screen_saver_dbus_screen_saver_complete_get_active (
-    skeleton, invocation, locked);
+  phosh_dbus_screen_saver_complete_get_active (skeleton, invocation, locked);
 
   return TRUE;
 }
 
 
 static gboolean
-handle_get_active_time (PhoshScreenSaverDBusScreenSaver *skeleton,
-                        GDBusMethodInvocation           *invocation)
+handle_get_active_time (PhoshDBusScreenSaver  *skeleton,
+                        GDBusMethodInvocation *invocation)
 {
   PhoshScreenSaverManager *self = PHOSH_SCREEN_SAVER_MANAGER (skeleton);
   guint delta = 0; /* in seconds */
@@ -134,15 +136,14 @@ handle_get_active_time (PhoshScreenSaverDBusScreenSaver *skeleton,
     delta = (g_get_monotonic_time () - active) / 1000000;
 
   g_debug ("DBus GetActiveTime: %u", delta);
-  phosh_screen_saver_dbus_screen_saver_complete_get_active_time (
-    skeleton, invocation, delta);
+  phosh_dbus_screen_saver_complete_get_active_time (skeleton, invocation, delta);
 
   return TRUE;
 }
 
 static gboolean
-handle_lock (PhoshScreenSaverDBusScreenSaver *skeleton,
-             GDBusMethodInvocation           *invocation)
+handle_lock (PhoshDBusScreenSaver  *skeleton,
+             GDBusMethodInvocation *invocation)
 {
   PhoshScreenSaverManager *self = PHOSH_SCREEN_SAVER_MANAGER (skeleton);
 
@@ -152,16 +153,15 @@ handle_lock (PhoshScreenSaverDBusScreenSaver *skeleton,
   g_debug ("DBus call lock");
   phosh_lockscreen_manager_set_locked (self->lockscreen_manager, TRUE);
 
-  phosh_screen_saver_dbus_screen_saver_complete_lock (
-    skeleton, invocation);
+  phosh_dbus_screen_saver_complete_lock (skeleton, invocation);
 
   return TRUE;
 }
 
 static gboolean
-handle_set_active (PhoshScreenSaverDBusScreenSaver *skeleton,
-                   GDBusMethodInvocation           *invocation,
-                   gboolean                         lock)
+handle_set_active (PhoshDBusScreenSaver  *skeleton,
+                   GDBusMethodInvocation *invocation,
+                   gboolean               lock)
 {
   PhoshScreenSaverManager *self = PHOSH_SCREEN_SAVER_MANAGER (skeleton);
 
@@ -177,15 +177,14 @@ handle_set_active (PhoshScreenSaverDBusScreenSaver *skeleton,
     g_debug ("Ignoring request to deactivate screen saver");
   }
 
-  phosh_screen_saver_dbus_screen_saver_complete_set_active (
-    skeleton, invocation);
+  phosh_dbus_screen_saver_complete_set_active (skeleton, invocation);
 
   return TRUE;
 }
 
 
 static void
-phosh_screen_saver_manager_screen_saver_iface_init (PhoshScreenSaverDBusScreenSaverIface *iface)
+phosh_screen_saver_manager_screen_saver_iface_init (PhoshDBusScreenSaverIface *iface)
 {
   iface->handle_get_active = handle_get_active;
   iface->handle_get_active_time = handle_get_active_time;
@@ -247,8 +246,7 @@ on_lockscreen_manager_wakeup_outputs (PhoshScreenSaverManager *self,
 
 
 static void
-on_logind_lock (PhoshScreenSaverManager            *self,
-                PhoshLogin1SessionDBusLoginSession *proxy)
+on_logind_lock (PhoshScreenSaverManager *self, PhoshDBusLoginSession *proxy)
 {
   g_debug ("Locking request via logind1");
   phosh_lockscreen_manager_set_locked  (self->lockscreen_manager, TRUE);
@@ -256,21 +254,22 @@ on_logind_lock (PhoshScreenSaverManager            *self,
 
 
 static void
-on_logind_unlock (PhoshScreenSaverManager            *self,
-                  PhoshLogin1SessionDBusLoginSession *proxy)
+on_logind_unlock (PhoshScreenSaverManager *self, PhoshDBusLoginSession *proxy)
 {
   g_debug ("Unlocking request via logind1");
   phosh_lockscreen_manager_set_locked  (self->lockscreen_manager, FALSE);
 }
 
 static void
-on_logind_prepare_for_sleep (PhoshScreenSaverManager            *self,
-                             gboolean                            suspending,
-                             PhoshLogin1ManagerDBusLoginManager *proxy)
+on_logind_prepare_for_sleep (PhoshScreenSaverManager *self,
+                             gboolean                 suspending,
+                             PhoshDBusLoginManager   *proxy)
 {
   g_return_if_fail (PHOSH_IS_SCREEN_SAVER_MANAGER (self));
 
-  if (!suspending) {
+  if (suspending) {
+    phosh_lockscreen_manager_set_locked (self->lockscreen_manager, TRUE);
+  } else {
     PhoshIdleManager *idle_manager;
 
     g_debug ("Got PrepareForSleep signal (resume case)");
@@ -281,6 +280,18 @@ on_logind_prepare_for_sleep (PhoshScreenSaverManager            *self,
   }
 }
 
+
+static void
+on_presence_status_changed (PhoshScreenSaverManager *self, guint32 status, gpointer *data)
+{
+  g_return_if_fail (PHOSH_IS_SCREEN_SAVER_MANAGER (self));
+
+  g_debug ("Presence status changed: %d", status);
+  if (status == PHOSH_SESSION_PRESENCE_STATUS_IDLE)
+    phosh_lockscreen_manager_set_locked (self->lockscreen_manager, TRUE);
+}
+
+
 static void
 on_name_acquired (GDBusConnection *connection,
                   const char      *name,
@@ -331,6 +342,9 @@ phosh_screen_saver_manager_dispose (GObject *object)
 {
   PhoshScreenSaverManager *self = PHOSH_SCREEN_SAVER_MANAGER (object);
 
+  g_cancellable_cancel (self->cancel);
+  g_clear_object (&self->cancel);
+
   g_clear_handle_id (&self->idle_id, g_source_remove);
   g_clear_handle_id (&self->dbus_name_id, g_bus_unown_name);
 
@@ -341,6 +355,8 @@ phosh_screen_saver_manager_dispose (GObject *object)
   g_clear_object (&self->logind_session_proxy);
   g_clear_object (&self->logind_manager_proxy);
 
+  g_clear_object (&self->presence);
+
   G_OBJECT_CLASS (phosh_screen_saver_manager_parent_class)->dispose (object);
 }
 
@@ -352,13 +368,15 @@ on_logind_get_session_proxy_finish  (GObject                 *object,
 {
   g_autoptr (GError) err = NULL;
 
-  self->logind_session_proxy = phosh_login1_session_dbus_login_session_proxy_new_for_bus_finish (
+  self->logind_session_proxy = phosh_dbus_login_session_proxy_new_for_bus_finish (
     res, &err);
   if (!self->logind_session_proxy) {
     phosh_dbus_service_error_warn (err, "Failed to get login1 session proxy");
-    goto out;
+    return;
   }
 
+  g_return_if_fail (PHOSH_IS_SCREEN_SAVER_MANAGER (self));
+
   /* finally register signals */
   g_object_connect (
     self->logind_session_proxy,
@@ -369,37 +387,34 @@ on_logind_get_session_proxy_finish  (GObject                 *object,
   g_signal_connect_swapped (
     self->logind_manager_proxy,
     "prepare-for-sleep", G_CALLBACK (on_logind_prepare_for_sleep), self);
-
-out:
-  g_object_unref (self);
 }
 
 
 static void
-on_logind_manager_get_session_finished (PhoshLogin1ManagerDBusLoginManager *object,
-                                        GAsyncResult                       *res,
-                                        PhoshScreenSaverManager            *self)
+on_logind_manager_get_session_finished (PhoshDBusLoginManager   *object,
+                                        GAsyncResult            *res,
+                                        PhoshScreenSaverManager *self)
 {
   g_autofree char *object_path = NULL;
-
   g_autoptr (GError) err = NULL;
 
-  if (!phosh_login1_manager_dbus_login_manager_call_get_session_finish (
+  if (!phosh_dbus_login_manager_call_get_session_finish (
         object, &object_path, res, &err)) {
     g_warning ("Failed to get session: %s", err->message);
-    goto out;
+    return;
   }
 
+  g_return_if_fail (PHOSH_IS_SCREEN_SAVER_MANAGER (self));
+
   /* Register a proxy for this session */
-  phosh_login1_session_dbus_login_session_proxy_new_for_bus (
+  phosh_dbus_login_session_proxy_new_for_bus (
     G_BUS_TYPE_SYSTEM,
     G_DBUS_PROXY_FLAGS_NONE,
     LOGIN_BUS_NAME,
-    object_path, NULL,
+    object_path,
+    self->cancel,
     (GAsyncReadyCallback)on_logind_get_session_proxy_finish,
-    g_object_ref (self));
-out:
-  g_object_unref (self);
+    self);
 }
 
 
@@ -411,31 +426,29 @@ on_logind_manager_proxy_new_for_bus_finish (GObject                 *source_obje
   g_autoptr (GError) err = NULL;
   g_autofree char *session_id = NULL;
 
-  g_return_if_fail (PHOSH_IS_SCREEN_SAVER_MANAGER (self));
-
   self->logind_manager_proxy =
-    phosh_login1_manager_dbus_login_manager_proxy_new_for_bus_finish (res, &err);
+    phosh_dbus_login_manager_proxy_new_for_bus_finish (res, &err);
 
   if (!self->logind_manager_proxy) {
     phosh_dbus_service_error_warn (err, "Failed to get login1 manager proxy");
-    goto out;
+    return;
   }
 
+  g_return_if_fail (PHOSH_IS_SCREEN_SAVER_MANAGER (self));
+
   /* If we find a session get it object path */
   if (phosh_find_systemd_session (&session_id)) {
     g_debug ("Logind session %s", session_id);
 
-    phosh_login1_manager_dbus_login_manager_call_get_session (
+    phosh_dbus_login_manager_call_get_session (
       self->logind_manager_proxy,
       session_id,
-      NULL,
+      self->cancel,
       (GAsyncReadyCallback)on_logind_manager_get_session_finished,
-      g_object_ref (self));
+      self);
   }
 
   g_debug ("Connected to logind's session interface");
-out:
-  g_object_unref (self);
 }
 
 
@@ -443,14 +456,14 @@ static gboolean
 on_idle (PhoshScreenSaverManager *self)
 {
   /* Connect to logind's session manager */
-  phosh_login1_manager_dbus_login_manager_proxy_new_for_bus (
+  phosh_dbus_login_manager_proxy_new_for_bus (
     G_BUS_TYPE_SYSTEM,
     G_DBUS_PROXY_FLAGS_NONE,
     LOGIN_BUS_NAME,
     LOGIN_OBJECT_PATH,
-    NULL,
+    self->cancel,
     (GAsyncReadyCallback) on_logind_manager_proxy_new_for_bus_finish,
-    g_object_ref (self));
+    self);
 
   self->idle_id = 0;
   return G_SOURCE_REMOVE;
@@ -475,6 +488,14 @@ phosh_screen_saver_manager_constructed (GObject *object)
 
   g_return_if_fail (PHOSH_IS_LOCKSCREEN_MANAGER (self->lockscreen_manager));
 
+  self->presence = phosh_session_presence_get_default_failable ();
+  if (self->presence) {
+    g_signal_connect_swapped (self->presence,
+                              "status-changed",
+                              (GCallback) on_presence_status_changed,
+                              self);
+  }
+
   /* Perform login1 setup when idle */
   self->idle_id = g_idle_add ((GSourceFunc)on_idle, self);
 }
@@ -504,24 +525,14 @@ phosh_screen_saver_manager_class_init (PhoshScreenSaverManagerClass *klass)
 static void
 phosh_screen_saver_manager_init (PhoshScreenSaverManager *self)
 {
+  self->cancel = g_cancellable_new ();
 }
 
 
 PhoshScreenSaverManager *
-phosh_screen_saver_manager_get_default (PhoshLockscreenManager *lockscreen_manager)
+phosh_screen_saver_manager_new (PhoshLockscreenManager *lockscreen_manager)
 {
-  static PhoshScreenSaverManager *instance;
-
-  if (instance == NULL) {
-    g_return_val_if_fail (PHOSH_IS_LOCKSCREEN_MANAGER (lockscreen_manager), NULL);
-    instance = g_object_new (PHOSH_TYPE_SCREEN_SAVER_MANAGER,
-                             "lockscreen-manager", lockscreen_manager,
-                             NULL);
-    g_object_add_weak_pointer (G_OBJECT (instance), (gpointer *)&instance);
-  } else {
-    /* switching lockscreen manager is not allowed */
-    g_return_val_if_fail (instance->lockscreen_manager == lockscreen_manager, instance);
-  }
-
-  return instance;
+  return g_object_new (PHOSH_TYPE_SCREEN_SAVER_MANAGER,
+                       "lockscreen-manager", lockscreen_manager,
+                       NULL);
 }
