@@ -676,7 +676,7 @@ setup_idle_cb (PhoshShell *self)
                            G_CONNECT_SWAPPED);
 
   /* Screen saver manager needs lock screen manager */
-  priv->screen_saver_manager = phosh_screen_saver_manager_get_default (
+  priv->screen_saver_manager = phosh_screen_saver_manager_new (
     priv->lockscreen_manager);
 
   priv->notify_manager = phosh_notify_manager_get_default ();
@@ -1305,6 +1305,19 @@ phosh_shell_get_toplevel_manager (PhoshShell *self)
 }
 
 
+PhoshScreenSaverManager *
+phosh_shell_get_screen_saver_manager (PhoshShell *self)
+{
+  PhoshShellPrivate *priv;
+
+  g_return_val_if_fail (PHOSH_IS_SHELL (self), NULL);
+  priv = phosh_shell_get_instance_private (self);
+
+  g_return_val_if_fail (PHOSH_IS_SCREEN_SAVER_MANAGER (self), NULL);
+  return priv->screen_saver_manager;
+}
+
+
 PhoshSessionManager *
 phosh_shell_get_session_manager (PhoshShell *self)
 {
@@ -1429,7 +1442,6 @@ phosh_shell_get_rotation_manager (PhoshShell *self)
   return priv->rotation_manager;
 }
 
-
 PhoshTorchManager *
 phosh_shell_get_torch_manager (PhoshShell *self)
 {
@@ -1805,9 +1817,10 @@ phosh_shell_set_state (PhoshShell          *self,
   str_new_flags = g_flags_to_string (PHOSH_TYPE_SHELL_STATE_FLAGS,
                                      priv->shell_state);
 
-  g_debug ("%s %s shells state. New state: %s",
-           enabled ? "Adding to" : "Removing from",
-           str_state, str_new_flags);
+  g_debug ("%s %s %s shells state. New state: %s",
+           enabled ? "Adding" : "Removing",
+           str_state,
+           enabled ? "to" : "from", str_new_flags);
 
   g_object_notify_by_pspec (G_OBJECT (self), props[PROP_SHELL_STATE]);
 
