@@ -23,6 +23,7 @@
 #include "osk-manager.h"
 #include "rotation-manager.h"
 #include "session-manager.h"
+#include "screen-saver-manager.h"
 #include "toplevel-manager.h"
 #include "torch-manager.h"
 #include "vpn-manager.h"
@@ -99,6 +100,7 @@ PhoshModeManager       *phosh_shell_get_mode_manager       (PhoshShell *self);
 PhoshMonitorManager    *phosh_shell_get_monitor_manager    (PhoshShell *self);
 PhoshToplevelManager   *phosh_shell_get_toplevel_manager   (PhoshShell *self);
 PhoshSessionManager    *phosh_shell_get_session_manager    (PhoshShell *self);
+PhoshScreenSaverManager *phosh_shell_get_screen_saver_manager (PhoshShell *self);
 /* Created on the fly */
 PhoshBtManager         *phosh_shell_get_bt_manager         (PhoshShell *self);
 PhoshDockedManager     *phosh_shell_get_docked_manager     (PhoshShell *self);
