@@ -55,12 +55,24 @@ phosh_shell_set_state (PhoshShell *self, guint state, gboolean set)
 {
 }
 
+guint
+phosh_shell_get_state (PhoshShell *self)
+{
+  return 0;
+}
+
 gboolean
 phosh_shell_get_locked (PhoshShell *self)
 {
   return FALSE;
 }
 
+gboolean
+phosh_shell_get_docked (PhoshShell *self)
+{
+  return FALSE;
+}
+
 PhoshAppTracker *
 phosh_shell_get_app_tracker (PhoshShell *self)
 {
